/**
 * Copyright 2014 Apoorv Parle <apparle@gmail.com>
 * Copyright 2015 David Edmundson <davidedmundson@kde.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License or (at your option) version 3 or any later version
 * accepted by the membership of KDE e.V. (or its successor approved
 * by the membership of KDE e.V.), which shall act as a proxy
 * defined in Section 14 of version 3 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include "findmyphoneplugin.h"

#include <QDBusConnection>
#include <QDebug>

#include <KPluginFactory>

K_PLUGIN_CLASS_WITH_JSON(FindMyPhonePlugin, "kdeconnect_findmyphone.json")

FindMyPhonePlugin::FindMyPhonePlugin(QObject* parent, const QVariantList& args) : KdeConnectPlugin(parent, args)
{
}

FindMyPhonePlugin::~FindMyPhonePlugin()
{
}

bool FindMyPhonePlugin::receivePacket(const NetworkPacket& np)
{
    Q_UNUSED(np);
    return false;
}

void FindMyPhonePlugin::ring()
{
    NetworkPacket np(PACKET_TYPE_FINDMYPHONE_REQUEST);
    sendPacket(np);
}

QString FindMyPhonePlugin::dbusPath() const
{
    return QString::fromLatin1("/modules/kdeconnect/devices/") + device()->id() + QString::fromLatin1("/findmyphone");
}

#include "findmyphoneplugin.moc"

